/************************************************************************/
/*                                                                      */
/*  MFLDEFS Header file                                                 */
/*                                                                      */
/*  Miscellaneous utility definitions for use with the MicroFirm        */
/*  Function Library                                                    */
/*                                                                      */
/*  Copyright 1989 by Robert B. Stout dba MicroFirm                     */
/*  All rights reserved                                                 */
/*                                                                      */
/*  Copyright 1986, 1987 by S.E. Margison                               */
/*                                                                      */
/*  Compiled by and for use with Zortech C/C++ 1.07 or later.           */
/*                                                                      */
/************************************************************************/

#ifndef MFLDEFS_H
#define MFLDEFS_H

/************************************************************************/
/* Miscellaneous #defines                                               */
/************************************************************************/

#define Repeat(x) if (x) do
#define Until(x) while(!(x))

#ifndef EVER
 #define EVER (;;)	/* e.g. "for EVER {...}"                        */
#elif __ZTC__ < 0x108
 #message "MFL warning: EVER already defined"
#endif

#define KB_flush() while(kbhit()) getch()
#define Bputc(c) bdos(6,c)
#define MAX_FLEN 67

/*
**  Use the following for "illegal" casts, e.g. structs or unions to
**  simple data types of the same size, etc.
*/

#define CAST(new_type,old_object) (*((new_type *)&old_object))

#if 0
*************************************************************************
*                                                                       *
*       /* Example of CAST macro at work        */                      *
*                                                                       *
*       union {                                                         *
*               char    ch[4];                                          *
*               int     i[2];                                           *
*       } my_union;                                                     *
*                                                                       *
*       long    longvar;                                                *
*                                                                       *
*               longvar = (long)my_union;       /* Illegal cast */      *
*               longvar = CAST(long, my_union); /* Legal cast   */      *
*                                                                       *
*************************************************************************
#endif  /* 0 */

#ifndef offsetof
 #define offsetof(st,el) (size_t)&(((st *)0)->el)
#elif __ZTC__ < 0x108
 #message "MFL warning: offsetof() already defined"
#endif

#ifndef NUL
 #define NUL '\000'
#elif __ZTC__ < 0x108
 #message "MFL warning: NUL already defined"
#endif

#ifndef BEL
 #define BEL '\007'
#elif __ZTC__ < 0x108
 #message "MFL warning: BEL already defined"
#endif

#ifndef EINVFNC
 #define EINVFNC 1
#elif __ZTC__ < 0x108
 #message "MFL warning: EINVFNC already defined"
#endif

#define __PAGE(title)

#ifdef TRUE
 #if __ZTC__ < 0x108
  #message "MFL warning: Redefining TRUE"
 #endif
 #undef TRUE
#endif

#ifdef FALSE
 #if __ZTC__ < 0x108
  #message "MFL warning: Redefining FALSE"
 #endif
 #undef FALSE
#endif

typedef enum {ERROR = -1, FALSE, TRUE} LOGICAL;

#ifndef BOOL
 #define BOOL(x) (!(!(x)))
#elif __ZTC__ < 0x108
 #message "MFL warning: BOOL already defined"
#endif

#ifndef SUCCESS
 #define SUCCESS 0	/* e.g. if (SUCCESS == fclose(...))     */
#elif __ZTC__ < 0x108
 #message "MFL warning: SUCCESS already defined"
#endif

#define BitSet(arg,posn) (arg | (1L << posn))
#define BitClr(arg,posn) (arg & ~(1L << posn))
#define BitTst(arg,posn) ((arg & (1L << posn)) ? TRUE : FALSE)

/* The following were present in DLC, but removed from ZTC      */

#ifndef min
 #define min(x,y) (((x) <= (y)) ? (x) : (y))
#elif __ZTC__ < 0x108
 #message "MFL warning: min() already defined"
#endif

#ifndef max
 #define max(x,y) (((x) >= (y)) ? (x) : (y))
#elif __ZTC__ < 0x108
 #message "MFL warning: max() already defined"
#endif

#define LP1 0    /* printer numbers */
#define LP2 1
#define LP3 2
#define SER1 0    /* COM1 port */
#define SER2 1    /* COM2 port */
#define SER3 2    /* COM3 port */
#define SER4 3    /* COM4 port */
#define NONE 0    /* no parity */
#define ODD 1     /* odd parity */
#define EVEN 2    /* even parity */

#define GAMEPORT 0x201        /* gameport address */
#define IDLE 0                /* state definitions for debouncer */
#define INPROG 1
#define WAITOPEN 2

#ifndef _Cdecl_
 #if __ZTC__ > 0x107
  #define CONST__ const
  #ifdef __STDC__
   #define _Cdecl_
  #else
   #define _Cdecl_ cdecl
  #endif
 #else
  #define CONST__
  #define _Cdecl_
 #endif
#endif

#if __ZTC__ > 0x107
 #if __cplusplus
  extern "C" {
 #endif
#endif

#undef MK_FP
void far * _Cdecl_ MK_FP(unsigned, unsigned);

#if __ZTC__ > 0x107
 #if __cplusplus
  }
 #endif
#endif

#endif /* MFLDEFS_H */
